// Incremental Saver v 2006.01.31
//
// Petter Sundnes
// petter@mentalfish.com
//
// This plugin comes with no warranty, use at own risk.
//
// Saves increments in a folder and does not overwrite any
// existing files. If you incremental save your file
// "myScene.lws", this script creates folder "myScene_incr"
// and adds the file "myScene_001.lws" to it.
//
// At the next incremental save it will add "myScene_002.lws",
// but you will still be working on the original "myScene.lws".
//
// This helps in having a clean incremental history of your 
// work as well as you will always know that "myScene.lws" is
// the actual latest save together with the highest number in
// the "myScene_incr" folder.
//
// The script looks for the highest numbered file in the _incr
// folder, and creates a new file, one number higher.
//
// If the file has not been saved yet, it will provide a file
// requester to specify a destination file.


@version 2.2
@warnings
@script generic
@name "MF Incremental Save"


//////
// Set your own preferences here
// (do not change in the middle of a project)

maxIncrements = 20; // Number of increments kept
folderExtension = "_incr_lws"; // Folder extension for increments
numberOfDigits = 3; // Number of digits to use "000" by default
versionSeperator = "_"; // Seperator of filename and version nr

//////

generic
{
  thisScene = Scene(); // Get a reference to the model
  currentPath = thisScene.filename; // Extract its full path
  if(currentPath == string("(unnamed)")){ 
    currentPath = getfile("Please save your scene first","*.lws","",0);
    if(currentPath == nil)
      error("Please save your model first:");
    if(strright(currentPath,4) != ".lws" && getsep() == "\\")
      currentPath = string(currentPath,".lws");
    createdFile = File(currentPath,"w");
  }
      
  if(strright(currentPath,4)==".lws")
    strLength = size(currentPath) - 4; // Strip the .lws extension
  else
    strLength = size(currentPath); // ..but not if it doesnt have  
  
  // Create the path to the incremental save folder:
  incrFolderPath = string(strleft(currentPath,strLength),folderExtension); 
  mkdir(incrFolderPath); // Create the folder

  nameArray = split(currentPath); 
  origFileName = nameArray[3]; // Retrieve the name of the file

  folderLength = 0;
  fileList = matchfiles(incrFolderPath, "*");
  folderLength = fileList.count();
  for(n=1;n<=fileList.count();n++){
    if(fileList[n]==".")
      folderLength = folderLength - 1;
    if(fileList[n] == "..")
      folderLength = folderLength - 1;
  }

  if(folderLength > 0){
    incrNum = fileList[fileList.count()];
    if(strright(fileList[fileList.count()],4)==".lws"){ 
      incrNum = strright(incrNum,4+numberOfDigits); // Strip .lws extension
    }
    else{
      incrNum = strright(incrNum,numberOfDigits); // If no .lws extension
    }
    incrNum = strleft(incrNum,numberOfDigits);
    incrNumInt = integer(incrNum);
    incrNum = incrNumInt + 1;
    incrNum = makeIncr(incrNum);
  }
  else {
    incrNum = makeIncr(1);
  }
  
  // If list is longer than maxIncrements, delete oldest entry
  if(folderLength >= maxIncrements){
    oldFile = string(incrFolderPath,getsep(),fileList[3]);
    filedelete(oldFile);
  }
    
  // Create a name suggestion:
  newFileName = string(origFileName,versionSeperator,incrNum,".lws");
  
  // Save an incremental version of the file:
  SaveSceneAs(string(incrFolderPath,getsep(),newFileName));
  
  // And overwrite the original file (work file):
  SaveSceneAs(currentPath);
  
}

makeIncr : intNum
{
  digitsPadding = "";
  for(n=1; n<=numberOfDigits; n++){
    digitsPadding = string(digitsPadding,"0");
  }
  numString = string(intNum);
  numLength = size(numString);
  threeDigits = string(strleft(digitsPadding,numberOfDigits - numLength),numString);
  return(threeDigits);
}