// Incremental Saver v 2006.01.31
//
// Petter Sundnes
// petter@mentalfish.com
//
// This plugin comes with no warranty, use at own risk.
//
// Saves increments in a folder and does not overwrite any
// existing files. If you incremental save your file
// "myModel.lwo", this script creates folder "myModel_incr"
// and adds the file "myModel_001.lwo" to it.
//
// At the next incremental save it will add "myModel_002.lwo",
// but you will still be working on the original "myModel.lwo".
//
// This helps in having a clean incremental history of your 
// work as well as you will always know that "myModel.lwo" is
// the actual latest save together with the highest number in
// the "myModel_incr" folder.
//
// The script looks for the highest numbered file in the _incr
// folder, and creates a new file, one number higher.
//
// If the file has not been saved yet, it will provide a file
// requester to specify a destination file.


@version 2.2
@warnings
@script modeler
@name "MF Incremental Save"


//////
// Set your own preferences here
// (do not change in the middle of a project)

maxIncrements = 20; // Number of increments kept
folderExtension = "_incr_lwo"; // Folder extension for increments
numberOfDigits = 3; // Number of digits to use "000" by default
versionSeperator = "_"; // Seperator of filename and version nr

//////

main
{
  thisModel = Mesh(0); // Get a reference to the model
  currentPath = thisModel.filename; // Extract its full path
  if(currentPath == nil){ 
    currentPath = getfile("Please save your model first","*.lwo","",0);
    if(currentPath == nil)
      error("Please save your model first:");
    if(strright(currentPath,4) != ".lwo" && getsep() == "\\")
      currentPath = string(currentPath,".lwo");
    createdFile = File(currentPath,"w");
  }
      
  if(strright(currentPath,4)==".lwo")
    strLength = size(currentPath) - 4; // Strip the .lwo extension
  else
    strLength = size(currentPath); // ..but not if it doesnt have  
  
  // Create the path to the incremental save folder:
  incrFolderPath = string(strleft(currentPath,strLength),folderExtension); 
  mkdir(incrFolderPath); // Create the folder

  nameArray = split(currentPath); 
  origFileName = nameArray[3]; // Retrieve the name of the file
  
  folderLength = 0;
  fileList = matchfiles(incrFolderPath, "*");
  folderLength = fileList.count();
  for(n=1;n<=fileList.count();n++){
    if(fileList[n]==".")
      folderLength = folderLength - 1;
    if(fileList[n] == "..")
      folderLength = folderLength - 1;
  }
  
  if(folderLength > 0){
    incrNum = fileList[fileList.count()];
    if(strright(fileList[fileList.count()],4)==".lwo"){ 
      incrNum = strright(incrNum,4+numberOfDigits); // Strip .lwo extension
    }
    else{
      incrNum = strright(incrNum,numberOfDigits); // If no .lwo extension
    }
    incrNum = strleft(incrNum,numberOfDigits);
    incrNumInt = integer(incrNum);
    incrNum = incrNumInt + 1;
    incrNum = makeIncr(incrNum);
  }
  else {
    incrNum = makeIncr(1);
  }
  
  // If list is longer than maxIncrements, delete oldest entry
  if(folderLength >= maxIncrements){
    oldFile = string(incrFolderPath,getsep(),fileList[3]);
    filedelete(oldFile);
  }
    
  // Create a name suggestion:
  newFileName = string(origFileName,versionSeperator,incrNum,".lwo");
  
  // Save an incremental version of the file:
  save(string(incrFolderPath,getsep(),newFileName));
  
  // And overwrite the original file (work file):
  save(currentPath);
  
}

makeIncr : intNum
{
  digitsPadding = "";
  for(n=1; n<=numberOfDigits; n++){
    digitsPadding = string(digitsPadding,"0");
  }
  numString = string(intNum);
  numLength = size(numString);
  threeDigits = string(strleft(digitsPadding,numberOfDigits - numLength),numString);
  return(threeDigits);
}